.tile-enerpi_data-power {
    background-image: radial-gradient(farthest-corner at 70% 70%, rgba(12, 187, 67, 0.83), rgba(55, 245, 119, 0.27));}
.tile-enerpi_data-power-text { color: #10371a; }
.tile-enerpi_data-kWh {
    background-image: radial-gradient(farthest-corner at 70% 70%, rgba(140, 39, 211, 0.83), rgba(191, 160, 245, 0.27));}
.tile-enerpi_data-kWh-text { color: #2c1037; }
.tile-enerpi_data-ldr {
    background-image: radial-gradient(farthest-corner at 70% 70%, rgba(244, 216, 63, 0.83), rgba(219, 221, 13, 0.33));}
.tile-enerpi_data-ldr-text { color: #333718; }

div.flex-img-container {
    background-color: rgba(13, 85, 35, 0.41);
    position: relative;
    top: 0;
    bottom: 0;
    left: 0;
    right: 0;
    display: flex; /* add */
    text-align: center;
    justify-content: center; /* add to align horizontal */
    align-items: center; /* add to align vertical */
}
img.hv-centered-image {
    vertical-align: middle;
    text-align: center;
    display: inline-block;
    height: auto;
    max-width: 65%;
	min-width: 35%;
}

/*.bk-root .bk-widget button .bk-bs-btn.bk-bs-btn-primary {*/
button.bk-bs-btn.bk-bs-btn-primary {
    border-color: #8C27D3 !important;
    border-width: 1px !important;
    background: rgba(151, 42, 227, 0.59) !important;
}

/*label.bk-bs-btn.bk-bs-btn-primary {*/
     /*display: none;*/
/*}*/

button.bk-bs-btn.bk-bs-btn-warning {
    border-color: #15619d !important;
    border-width: 1px !important;
    background: #2196F3 !important;
}

.bk-plot-above.bk-toolbar-active {
    border-bottom-color: #10371a;
    border-width: 3px;
}

.bk-tooltip-color-block {
    outline: None;
}

.bk-tooltip.bk-tooltip-custom {
    /*border-color: #0CBB43;*/
    /*border-color: #098b32;*/
    border-color: #15619d !important;
    /*background: linear-gradient(rgba(55, 245, 119, 0.27), rgba(233, 232, 235, 0.71));*/
    /*background: rgba(55, 245, 119, 0.27) !important;*/
    border-width: 2px;
}

.classif-used-tag {
    border-color: #15619d !important;
    border-width: 2px !important;
    background: #2196F3 !important;
    /*background: #f4af38 !important;*/
    /*background: #F4D83F !important;*/
    /*background: #8C27D3 !important;*/
    /*background: #972625 !important;*/
    /*background: #be1412 !important;*/
}
.classif-unused-tag {
    border-color: rgba(30, 138, 222, 0.82) !important;
    border-width: 1px !important;
    background: rgba(35, 158, 255, 0.45) !important;
}

.classif-assigned-tag {
    border-color: rgba(12, 187, 67, 0.83) !important;
    border-width: 2px !important;
    background: rgba(55, 245, 119, 0.27) !important;
}
/*.navbar-brand {*/

    /*padding-bottom: -1px; !important;*/
    /*margin-bottom: -1px; !important;*/
/*}*/
.tile-monitor-container {
    display: flex; /* or inline-flex */
    flex-flow: row wrap;
    justify-content: space-around;
    align-items: flex-start;
    align-content: flex-start;
    overflow: hidden;
}
.tile-monitor-container-join {
    margin-left: -1px;
    /*margin-top: -100px;*/
    display: flex; /* or inline-flex */
    flex-flow: row wrap;
    justify-content: space-around;
    align-items: flex-start;
    align-content: flex-start;
    overflow: hidden;
}
.tile-monitor {
    width: 480px;
    height: 200px;
    flex-shrink: .7;
    flex-grow: 1.6;
    min-width: 300px;
    max-width: 900px;
}
.tile-img-bg {
    position: relative;
    width: 100%;
    height: 100%;
    /*-webkit-transition: background 2s linear;*/
    /*transition: background 2s linear;*/
    /*background-size: 100% 100%;*/
    /*background-repeat: no-repeat;*/
}
.tile-monitor-label {
    position: absolute;
    text-align: left;
    font-family:'Inconsolata',sans-serif;
    font-weight: 200;
    font-size:12px;
    margin: 7px 7px 0;
    line-height: 0.95;
}
.tile-monitor-label-sub {
    font-size:10px;
}
.tile-monitor-value {
    font-family:'Inconsolata',sans-serif;
    font-weight:700;
    font-size: 140px;
    letter-spacing: -5px;
    margin:10px 10px;
}
.tile-monitor-unit {
    font-family:'Open Sans',Arial,sans-serif;
    font-weight:100;
    font-size:25px;
    margin:10px 10px 25px 10px;
}
.tile-monitor-innerContent {
    position: relative;
    margin-top: -200px;
}
.tile-monitor-innerContent > span > .tile-monitor-unit {
    text-align: right;
    line-height: 0.75;
}
.tile-monitor-innerContent > span > .tile-monitor-value {
    text-align: right;
    line-height: 0.75;
    text-shadow: 2px 2px 4px rgba(50, 37, 31, 0.35);
}

.minipulse{
	animation-name: minipulse;
	-webkit-animation-name: minipulse;

	animation-duration: 200ms;
	-webkit-animation-duration: 200ms;

	animation-iteration-count: 1;
	-webkit-animation-iteration-count: 1;
}
@keyframes minipulse {
	0% {
		transform: scale(1);
		opacity: 1;
	}
	25% {
		transform: scale(1.05);
		opacity: 0.9;
	}
	50% {
		transform: scale(1.1);
		opacity: 0.8;
	}
	75% {
		transform: scale(1.05);
		opacity: 0.9;
	}
	100% {
		transform: scale(1);
		opacity: 1;
	}
}

@-webkit-keyframes minipulse {
	0% {
		transform: scale(1);
		opacity: 1;
	}
	25% {
		transform: scale(1.05);
		opacity: 0.9;
	}
	50% {
		transform: scale(1.1);
		opacity: 0.8;
	}
	75% {
		transform: scale(1.05);
		opacity: 0.9;
	}
	100% {
		transform: scale(1);
		opacity: 1;
	}
}
