{% macro alert_dismiss(obj_alert) %}
    <div class="container-fluid pb-0 mb-0">
        <div class="alert alert-{{ obj_alert.alert_type }} alert-dismissable"
             style="word-wrap:break-word; overflow-wrap: break-word;">
            <button type="button" class="close" data-dismiss="alert"
                aria-hidden="true">
                &times;
            </button>
            {{ obj_alert.texto_alerta |safe }}
        </div>
    </div>
{% endmacro %}

{% macro style_class_greyscale(name, perc) %}
<style type="text/css">
    .{{ name }} {
        filter: none;
        -webkit-filter: grayscale({{ perc }}%);
        -moz-filter: grayscale({{ perc }}%);
        -ms-filter: grayscale({{ perc }}%);
        -o-filter: grayscale({{ perc }}%);
    }
</style>
{% endmacro %}

{% macro styles_bokeh(version) %}
    <link rel="stylesheet" href="http://cdn.pydata.org/bokeh/release/bokeh-{{ version }}.min.css" type="text/css"/>
    <link rel="stylesheet" href="http://cdn.pydata.org/bokeh/release/bokeh-widgets-{{ version }}.min.css" type="text/css"/>
    <script type="text/javascript" src="http://cdn.pydata.org/bokeh/release/bokeh-{{ version }}.min.js"></script>
    <script type="text/javascript" src="http://cdn.pydata.org/bokeh/release/bokeh-widgets-{{ version }}.min.js"></script>
{% endmacro %}
