{% extends "layout.html" %}
{#{% from 'monitor_tiles.html' import monitor_tiles with context %}#}
{#{% from 'macros.html' import styles_bokeh with context %}#}

{% block content %}
    <div class="jumbotron">
        <div class="container-fluid">
            <div class="row">
                <div class="col-xs-3 col-sm-2 col-md-1 col-lg-1">
                    <img class="img-fluid img-rounded pull-xs-left" style="max-width: 100%; height: auto; width: 100% \9;"
                         src="{{ url_for('static', filename='img/icons/base.svg')}}">
                </div>
                <div class="col-xs-6 col-sm-8 col-md-9 col-lg-9">
                    <h1 class="display-5"><i class="fa fa-tags" aria-hidden="true"></i> Manual confirmation</h1>
{#                    <p>Start labeling your appliances</p>#}
                </div>
            </div>
        </div>
    </div>
    <div class="container-fluid">
        <div class="bk-root">
          {{ bokeh_script|safe }}
        </div>
    </div>
{#    <div class="col-xs-12 col-sm-8 col-md-9 col-lg-6">#}
{#        <br class="hidden-sm hidden-xs">#}
{#        <a class="btn btn-secondary btn-outline-primary" href="{{ url_for('showfile', file='flask', tail=200, reverse=True)}}"#}
{#           role="button"><i class="fa fa-arrow-left" aria-hidden="true"></i> Anterior</a>#}
{#        <a class="btn btn-secondary btn-outline-warning" href="{{ url_for('showfile', file='flask', tail=200, reverse=True)}}"#}
{#           role="button"><i class="fa fa-calendar" aria-hidden="true"></i></a>#}
{#        <a class="btn btn-secondary btn-outline-success" href="{{ url_for('showfile', file='rsc', tail=200, reverse=True)}}"#}
{#           role="button"><i class="fa fa-check" aria-hidden="true"></i> Validar</a>#}
{#        <a class="btn btn-secondary btn-outline-primary" href="{{ url_for('showfile', file='rsc', tail=200, reverse=True)}}"#}
{#           role="button">Siguiente <i class="fa fa-arrow-right" aria-hidden="true"></i></a>#}
{#    </div>#}

{% endblock %}
