{% extends "layout.html" %}
{#{% from 'monitor_tiles.html' import monitor_tiles with context %}#}
{#{% from 'macros.html' import styles_bokeh with context %}#}

{% block content %}
    <div class="jumbotron">
        <div class="container-fluid">
            <div class="row">
                <div class="col-xs-6 col-sm-4 col-md-3 col-lg-3">
                    <img class="img-fluid img-rounded pull-xs-left" style="max-width: 100%; height: auto; width: 100% \9;"
                         src="{{ url_for('static', filename='img/icons/base.svg')}}">
                </div>
                <div class="col-xs-6 col-sm-8 col-md-9 col-lg-9">
                    <h2 class="display-5">EnerPi Appliances Recognition</h2>
                </div>
                <div class="col-xs-12 col-sm-8 col-md-9 col-lg-6">
                    <br class="hidden-sm hidden-xs">
                    <a class="btn btn-secondary btn-outline-primary" href="{{ url_for('manual_tagging')}}"
                       role="button"><i class="fa fa-tags" aria-hidden="true"></i> Manual labeling</a>
                    {% if show_confirmation_page %} }}
                        <a class="btn btn-secondary btn-outline-primary" href="{{ url_for('manual_confirmation')}}"
                           role="button"><i class="fa fa-check-circle" aria-hidden="true"></i> Manual confirmation</a>
                    {% else %}
                        <a class="btn btn-secondary btn-outline-primary" disabled href="{{ url_for('manual_confirmation')}}"
                           role="button"><i class="fa fa-check-circle" aria-hidden="true"></i> Manual confirmation</a>
                    {% endif %}
                </div>
            </div>
        </div>
    </div>
{% endblock %}
